#!/bin/bash
# Atomic Secured Linux 
# Copyright Atomicorp, 2007-2014
# License: Commercial, unauthorized redistribution or duplication is prohibited
# Name: configuration_setup.sh



source "/var/asl/lib/functions"
source "/etc/asl/config"

if [ -f /etc/asl/license.key ]; then 
	LK_MODE=1
else
	LK_MODE=0
fi

if [ -f /etc/asl/tortix.key ]; then
	LK_MODE=0
fi

if [ ! $SSH_TTY ]; then
	SSH_TTY="/dev/$(ps -p$$ --no-heading | awk '{print $2}')"
fi

if [ "$1" == "auto" ]; then
	AUTO=auto
fi



# Fixup
HOSTNAME=`/bin/hostname`
perl -pi -e "s/^HOSTNAME=.*/HOSTNAME=\"$HOSTNAME\"/" /etc/asl/config

originalSettings=`stty -g -F $SSH_TTY`
function fflush_stdin() {
    stty -icanon min 0 time 0 -F $SSH_TTY
    while read dummy ; do : ; done
    stty "$originalSettings" -F $SSH_TTY
}


function isValid() {
  compressed="$(echo $1 | egrep -q '^[[:alnum:]]+$')"
  return
}


function yum_config_manager() {
	if [ -f /etc/yum.repos.d/tortix-kernel.repo ]; then
	  	sed -i 's/^enabled.*/enabled = 0/g' /etc/yum.repos.d/tortix-kernel.repo
	  	if [ "$KERNEL_CHANNEL" == "tortix-kernel" ]; then
			sed -i ':a;N;$!ba;s/enabled = 0/enabled = 1/1' /etc/yum.repos.d/tortix-kernel.repo
	  	elif [ "$KERNEL_CHANNEL" == "tortix-kernel-xen" ] ; then
			sed -i ':a;N;$!ba;s/enabled = 0/enabled = 1/3' /etc/yum.repos.d/tortix-kernel.repo
	  	fi
	fi

}

if [ "$1" == "yum_config_manager" ]; then
	yum_config_manager
	exit
fi


function appinv_check {
	# If ASL hasn't been configured before, update a couple of things that can go
	# stale while they were packaged.
	if [ "$CONFIGURED" == "no" ]; then

	  # run App inventory 
	  echo
	  echo "Web Application Inventory"
	  echo "  ASL will now scan your system for web based applications it can identify"
	  echo "  This index will be used to configure other components of the system, and"
	  echo "  will be added to a nightly cron job to keep the index up to date."
	  echo "  This index is located in /var/asl/data/webapp.db, and can be viewed through"
	  echo "  the ASL web interface."
	  echo "  Note: This process can take a very long time, and will run in the background"
	  $ASLHOME/lib/modules/webapp_inventory_check.sh $FIXMODE > /dev/null  2>&1 &

	fi
}

# changepass <msg> <alpha y/n> <blank y/n>
# returns password

function changepass {

  msg=$1
  alpha=$2
  blank=$3

  # Prompt the user for the desired password and verify its accuracy.  
  failed=0
  PASSCONFIRMED=0
  while [ $PASSCONFIRMED -lt 1 ]; do
    	if [ $failed -gt 2 ]; then
      		echo "Exiting: too many failed attempts." |tee -a $LOG
      		echo
      		exit 1
    	fi

        echo -n "$msg: "
        unset PASSWORD
        while IFS= read -r -s -n1 pass <$SSH_TTY ; do
                if [[ -z $pass ]]; then
                        echo
                        break
                else
                        echo -n '*'
                        PASSWORD+=$pass
                fi
        done



    	if [ "$PASSWORD" == "" ] && [ "$blank" == "no" ]; then
      		echo "Exiting: Password is blank..." | tee -a $LOG
      		echo 
      		exit 1
    	fi

    	if [ "$alpha" == "yes" ] && ! isValid "$PASSWORD"; then
       		echo "Exiting: Only Alpha-Numeric passwords are supported..." | tee -a $LOG
        	echo
        	exit 1
  	fi

        unset PASSWORD2
        echo -n "Re-$msg: "
        while IFS= read -r -s -n1 pass <$SSH_TTY; do
                if [[ -z $pass ]]; then
                        echo
                        break
                else
                        echo -n '*'
                        PASSWORD2+=$pass
                fi
        done


    	if [ "$PASSWORD" == "$PASSWORD2" ]; then
      		PASSCONFIRMED=1
    	else
      		failed=$(( $failed + 1 ))
      		echo "Sorry, passwords do not match." | tee -a $LOG
      		echo
    	fi

  done


}

function update_policy {
  # Update policy
  if [ $LK_MODE -lt 1 ]; then 
	  echo 
	  echo "Update Policy"
	  echo "  Aum has the ability to download updates for rules and packages"
	  echo "  installed on the system automatically. You can select from three"
	  echo "  different types of automatic updates, or disable automatic updates completely."
	  echo
	  echo "  All - Update both rules and ASL packages automatically"
	  echo "  Rules-Only - update just WAF, HIDS, Geo-Map, and Malware rules"
	  echo "  Exclude-kernel - Update all packages, excluding the ASL kernel"
	  echo "  Off - Disable all automatic updates"
	  echo
	  if [ ! $AUTO ]; then
		if [ $ASL -ge 1 ]; then
			check_input "  Update Type? (all/rules-only/exclude-kernel/off) [Default: $UPDATE_TYPE]:" "all|rules-only|exclude-kernel|off" $UPDATE_TYPE
			UPDATE_TYPE=$INPUTTEXT
		else
			check_input "  Update Type? (all/rules-only/off) [Default: $UPDATE_TYPE]:" "all|rules-only|off" $UPDATE_TYPE
			UPDATE_TYPE=$INPUTTEXT
			FEED_TYPE="lite"
			UPDATE_PATH="/channels/rules/"
			perl -pi -e "s[UPDATE_PATH=.*][UPDATE_PATH=\"$UPDATE_PATH\"]g" /etc/asl/config
		fi
		check_input "  Update Frequency? (none/hourly/daily) [Default: $AUTOMATIC_UPDATES]:" "none|hourly|daily" $AUTOMATIC_UPDATES
	  	AUTOMATIC_UPDATES=$INPUTTEXT
	  fi
	  perl -pi -e "s/^UPDATE_TYPE=.*/UPDATE_TYPE=\"$UPDATE_TYPE\"/"  /etc/asl/config
	  perl -pi -e "s/^FEED_TYPE=.*/FEED_TYPE=\"$FEED_TYPE\"/"  /etc/asl/config
	  perl -pi -e "s/^AUTOMATIC_UPDATES=.*/AUTOMATIC_UPDATES=\"$AUTOMATIC_UPDATES\"/"  /etc/asl/config
  else 
	perl -pi -e "s/^UPDATE_TYPE=.*/UPDATE_TYPE=\"all\"/"  /etc/asl/config
  fi
}

function kernel_update {
  if  [ "$UPDATE_TYPE" == "exclude-kernel" ]; then
		KERNEL_CHANNEL="disabled"

  else 

	  echo
	  echo
	  echo "Kernel Update and Virtualization settings"
	  echo 
	  echo "  The ASL kernel includes extensive advanced security features including"
          echo "    * Real-time malware detection"
          echo "    * Active kernel intrusion prevention"
          echo "    * Advanced firewall capabilities"
	  echo
	  echo "  Certain ASL settings need to be modified including firewall"
	  echo "  functionality in Virtuozzo/Openvz, and compatiblity modifications"
	  echo "  virtualization environments like xen."
	  echo

	  if [ ! $AUTO ]; then

		if rpm --quiet -q trixbox ; then
			echo
			echo "		Trixbox detected, skipping kernel installation"
			echo
			KERNEL_CHANNEL="disabled"
		elif rpm --quiet -q freepbx ; then
			echo
			echo "		Freepbx detected, skipping kernel installation"
			echo
			KERNEL_CHANNEL="disabled"	
		elif [ "$DIST" == "el7" ]; then
			echo
			echo "		EL7 detected, skipping kernel installation"
			echo
			KERNEL_CHANNEL="disabled"
		elif [ "$DIST" != "el7" ] && [ ! -f /sbin/grub ];then
			echo
			echo "		Grub (/sbin/grub) not detected, skipping kernel installation"
			echo
			KERNEL_CHANNEL="disabled"
		elif [ "$DIST" == "el7" ] && [ ! -f /boot/grub2/grub.cfg ]; then
			echo
			echo "		Grub (/boot/grub2/grub.cfg) not detected, skipping kernel installation"
			echo
			KERNEL_CHANNEL="disabled"
		else

			# Virtuozzo
			if [ -f /proc/vz/veinfo ] ; then
				K_DEFAULT="virtuozzo"
			elif [ -d /proc/xen -o -d /proc/sys/xen ]; then
				K_DEFAULT="xen"

				echo
				echo "Xen Detected..."
				echo
				echo "Please note, installation of xen kernels can "
				echo "only be supported through package management (RPM)."
				echo "Non-package managed environments are not supported."
				echo


			else 
				K_DEFAULT="none"
			fi

		  check_input "  Virtualization Type? (none/virtuozzo/openvz/xen/vmware/kvm/hyper-v) [Default: $K_DEFAULT]" "none|virtuozzo|xen|openvz|vmware|kvm|hyper-v" $K_DEFAULT
		  if [ "$INPUTTEXT" == "virtuozzo" -o "$INPUTTEXT" == "openvz" ]; then
			KERNEL_CHANNEL="disabled"
			FW_DROP_INVALID="no"
			echo "G,hids,4151,,yes,no,0,no,no," >> /etc/asl/rules
		  elif [ "$INPUTTEXT" == "xen" ]; then

			echo
			echo "  Xen selected."
			echo

			unset INPUTTEXT
			check_input "  Do you understand that only RPM package managed environments can be supported? (yes/no) [Default: yes]" "yes|no" "yes"
			if [ "$INPUTTEXT" == "no" ]; then
				KERNEL_CHANNEL="disabled"
				FW_DROP_INVALID="yes"
			else
				KERNEL_CHANNEL="tortix-kernel-xen"
				FW_DROP_INVALID="yes"
			fi
		  else
			KERNEL_CHANNEL="tortix-kernel"
			FW_DROP_INVALID="yes"
		  fi
		fi

	  fi
  fi



  perl -pi -e "s/^FW_DROP_INVALID=.*/FW_DROP_INVALID=\"$FW_DROP_INVALID\"/"  /etc/asl/config
  perl -pi -e "s/^KERNEL_CHANNEL=.*/KERNEL_CHANNEL=\"$KERNEL_CHANNEL\"/"  /etc/asl/config

  yum_config_manager

}

function core_settings {
	echo 
	echo
	echo "General Settings"
	echo "  ASL can be configured as a central Web management console,  "
	echo "  log and event collection for the local and remote systems, or"
	echo "  as a client, reporting events to a remote ASL server."
	echo

	if [ $LK_MODE -ge 1 ]; then 
		OSSEC_MODE="server"
	elif [ ! $AUTO ]; then
		# Define the mode in which ASL will operate.
		check_input "  ASL mode? (server/client) [Default: $OSSEC_MODE]:" "server|client" $OSSEC_MODE
		OSSEC_MODE=$INPUTTEXT
	fi

  	# Modify the ASL config file with the chosen OSSEC setting.
	perl -pi -e "s/^OSSEC_MODE=.*/OSSEC_MODE=\"$OSSEC_MODE\"/" /etc/asl/config 

	if [ "$OSSEC_MODE" == "server" ]; then

		if [ "$DIST" == "el7" ]; then
			systemctl enable mariadb.service > /dev/null 2>&1

			service mariadb status > /dev/null 2>&1
			RETVAL=$?
			if [ $RETVAL -ge 1 ]; then
				echo "  Starting MariaDB, this is an ASL Web dependency"
				systemctl start mariadb.service > /dev/null 2>&1
			fi

		else

			/sbin/chkconfig mysqld on > /dev/null 2>&1

			/etc/init.d/mysqld status > /dev/null 2>&1
			RETVAL=$?
			if [ $RETVAL -ge 1 ]; then
				echo "  Starting MySQL, this is an ASL Web dependency"
				/sbin/service mysqld start > /dev/null 2>&1
			fi 
		fi
		
		# Install ASL Web
    		if ! rpm --quiet -q asl-web ; then
			echo "  ASL Web Console not detected. Installation will take a few moments."
			echo -n "  Installing ASL Web Console: "
			/usr/bin/yum --disableexcludes=all -y install asl-web >/dev/null
			RETVAL=$?
			if [ $RETVAL -ge 1 ]; then
				echo "Failed."
				echo "  Error: ASL Web Console could not be installed. Configuration halted."
				exit 1
			else
				# ASLW key file
				if ! [ -f /var/asl/data/aslw.key ]; then
					touch /var/asl/data/aslw.key
				fi
				chmod 0600 /var/asl/data/aslw.key
				chown tortix:root /var/asl/data/aslw.key
				# ASLW scan log file
				if ! [ -f /etc/asl/aslw_scan.log ]; then
					touch /etc/asl/aslw_scan.log
				fi
				chmod 0660 /etc/asl/aslw_scan.log
				chown tortix:root /etc/asl/aslw_scan.log
				# ASLW update log file
				if ! [ -f /etc/asl/aslw_aum.log ]; then
					touch /etc/asl/aslw_aum.log
				fi
				chmod 0660 /etc/asl/aslw_aum.log
				chown tortix:root /etc/asl/aslw_aum.log
				# ASLW clamdscan log file
				if ! [ -f /etc/asl/aslw_malware_scan.log ]; then
					touch /etc/asl/aslw_malware_scan.log
				fi
				chmod 0660 /etc/asl/aslw_malware_scan.log
				chown tortix:root /etc/asl/aslw_malware_scan.log
				echo "Complete"
			fi

		fi
		
		# Remove client if detected
		if rpm --quiet -q ossec-hids-client ; then
			echo -n "  Client install detected. Removing client package: "
			yum -y remove  ossec-hids-client >/dev/null
			RETVAL=$?
			if [  $RETVAL -ge 1 ]; then
				echo "Failed."
				echo "  Error: ASL client could not be removed. Configuration halted."
				exit 1
			else
				echo "Complete"
				
			fi
		fi
  
    		# If the OSSEC server component isn't installed, use yum to install it.
    		if  ! rpm --quiet -q ossec-hids-server ; then
      			# stop psmon
      			/etc/init.d/psmon stop >/dev/null 2>&1
      			echo
      			echo -n "  Installing ASL server: "
      			yum -y install ossec-hids-server >/dev/null 
			RETVAL=$?
			if [ $RETVAL -ge 1 ]; then
				echo "Failed."
				echo "  Error: ASL server could not be installed. Configuration halted."
				exit 1
			else
				echo "Complete"
			fi
      			/etc/init.d/psmon start >/dev/null 2>&1
    		fi


    		# Define the mode in which OSSEC will operate.
    		OSSEC_USE_MYSQL="yes"

    		# Modify the ASL config file with the chosen OSSEC setting.
    		perl -pi -e "s/^OSSEC_USE_MYSQL=.*/OSSEC_USE_MYSQL=\"$OSSEC_USE_MYSQL\"/" /etc/asl/config 

		if [ $LK_MODE -lt 1 ]; then 
			echo
			echo "  Advanced Database settings allow for configurating alternate"
			echo "  database names, Remote mysql servers, and user custom credentials."
			echo "  If you do not know, select no"
			echo
	
			if [ ! $AUTO ]; then
	  			check_input "  Advanced database configuration? (yes/no) [Default: no ]:" "yes|no" no
			else
				INPUTTEXT=no
			fi
		
			export LANG=C
			RANDPASS=$(tr -dc "[:alnum:]" < /dev/urandom 2>/dev/null| head -c 8)
	
			if [ "$INPUTTEXT" == "no" ]; then
				OSSEC_DATABASE_SERVER=127.0.0.1
				OSSEC_DATABASE=tortix
				OSSEC_DATABASE_USERNAME=tortix
				if [ "$OSSEC_DATABASE_PASSWORD" == "" ]; then
					OSSEC_DATABASE_PASSWORD=$RANDPASS
				fi
			else
		
				# If mysql is enabled, continue dialog for the database server
				O_SERVER_CONFIRMED=0
				while [ $O_SERVER_CONFIRMED -lt 3 ]; do
					if [ $O_SERVER_CONFIRMED -gt 2 ]; then
						echo "	Error: too many failed connection attempts."
						echo
						break
					fi
	
					if [ ! $AUTO ]; then
						echo -n "  Mysql Server [Default: $OSSEC_DATABASE_SERVER]: " 
						read O_DATABASE_SERVER < $SSH_TTY
					fi
	
					if [ "$O_DATABASE_SERVER" != "" ]; then
						OSSEC_DATABASE_SERVER=$O_DATABASE_SERVER
					fi
	
					ping -c 2 $OSSEC_DATABASE_SERVER > /dev/null
					if [ $? -eq  1 ] ; then
						echo "	Testing network connection to database server ($OSSEC_DATABASE_SERVER): failed"
						echo "	Resetting setting to default."
						OSSEC_DATABASE_SERVER="127.0.0.1"
						O_SERVER_CONFIRMED=$(($O_SERVER_CONFIRMED + 1))
					else
						echo "	Testing network connection to database server ($OSSEC_DATABASE_SERVER): success"
						break
					fi
					
				done		
	
				fflush_stdin
	
				# Database name
				if [ ! $AUTO ]; then
					echo -n "  Database name [Default: $OSSEC_DATABASE]: " 
					read O_DATABASE < $SSH_TTY
				fi
				if [ "$O_DATABASE" != "" ]; then
					OSSEC_DATABASE=$O_DATABASE
				fi
		    
				# Database user
				O_USER_CONFIRMED=0
				while [ $O_USER_CONFIRMED -lt 3 ]; do
					if [ $O_USER_CONFIRMED -gt 2 ]; then
						echo "	Error: too many failed username attempts."
						echo
						break
					fi
			
					if [ ! $AUTO ]; then
						echo -n "  Database username [Default: $OSSEC_DATABASE_USERNAME]: "
						read O_DATABASE_USERNAME < $SSH_TTY
					fi
	
					if [ "$O_DATABASE_USERNAME" != "" ] ; then 
						OSSEC_DATABASE_USERNAME=$O_DATABASE_USERNAME
					fi
					if [ "$O_DATABASE_USERNAME" == "root" ] ; then 
						echo "	Username root is reserved. Resetting selection to tortix."
						OSSEC_DATABASE_USERNAME=tortix
						O_USER_CONFIRMED=$(($O_USER_CONFIRMED + 1))
					else
						break
					fi
				done
				echo
	
	
				# Database password
				if [ "$OSSEC_DATABASE_PASSWORD" == "" -o "$OSSEC_DATABASE_PASSWORD" == "OSSEC_ASL_OSSEC" ]; then
					if [ ! $AUTO  ]; then
						echo -n "  Database password [Default: generated]: "
						read -s OSSEC_DATABASE_PASSWORD < $SSH_TTY
						echo
					#	while IFS= read -r -s -n1 pass <$SSH_TTY ; do	
					#		if [[ -z $pass ]]; then
					#			echo
					#			break
					#		else
					#			echo -n '*'
					#			OSSEC_DATABASE_PASSWORD+=$pass
					#		fi
					#
					#	done
					fi 
	
	
					if [ "$OSSEC_DATABASE_PASSWORD" == "" ]; then
						OSSEC_DATABASE_PASSWORD=$RANDPASS
					fi
				else
					echo "  Database password is already set"
				fi
				echo
			fi

            perl -pi -e "s/^OSSEC_DATABASE_SERVER=.*/OSSEC_DATABASE_SERVER=\"$OSSEC_DATABASE_SERVER\"/"  /etc/asl/config
            perl -pi -e "s/^OSSEC_DATABASE=.*/OSSEC_DATABASE=\"$OSSEC_DATABASE\"/"  /etc/asl/config
            perl -pi -e "s/^OSSEC_DATABASE_USERNAME=.*/OSSEC_DATABASE_USERNAME=\"$OSSEC_DATABASE_USERNAME\"/"  /etc/asl/config
            perl -pi -e "s/^OSSEC_DATABASE_PASSWORD=.*/OSSEC_DATABASE_PASSWORD=\"$OSSEC_DATABASE_PASSWORD\"/"  /etc/asl/config

			# Check for database
			if [ ! $AUTO ]; then
   				/var/asl/bin/database-setup 
			else
      				/var/asl/bin/database-setup  auto
				if [ $? -ge 1 ]; then
					echo "Error: database-setup exited with $?"
				fi
			fi

			# Query cache check
			if grep -q query_cache_size /etc/my.cnf; then
				QUERY_CACHE_SIZE=`awk -F= '/query_cache_size/ { print $2}' /etc/my.cnf`
    				perl -pi -e "s/^MYSQL_QUERY_CACHE=.*/MYSQL_QUERY_CACHE=\"$QUERY_CACHE_SIZE\"/" /etc/asl/config

			fi

      		# end mysql settings
		fi

		# E-mail notifications: This is picked up everywhere.
		echo "Notifications"
		echo "  ASL will send notifications to the users defined below. These notifications"
		echo "  include security incidents, anomalous behavior, changes to the system"
		echo "  and general diagnostic information"
		echo
  
		if [ "$EMAIL" == "" ]; then
			EMAIL="root@localhost"
		fi

		if [ ! $AUTO ] && [ $LK_MODE -lt 1 ]; then
			echo -n "  Email address for notifications [Default: $EMAIL]: "
	 		read ADMIN_MAIL < $SSH_TTY
		fi

  		if [ "$ADMIN_MAIL" != "" ]; then
    			EMAIL=$ADMIN_MAIL
  		fi  

    		# Notifications per hour
		INPUTTEXT=$OSSEC_MAX_MSG 
		if [ ! $AUTO ]; then
    			echo -n "  Max Email notifications per hour? (1-9999) [Default: $OSSEC_MAX_MSG ]: " 
    			read INPUTTEXT < $SSH_TTY
		fi
	
    		compressed="$(echo $INPUTTEXT | sed -e 's/[^[:digit:]]//g')"
    		if [ "$compressed" != "$INPUTTEXT" -o "$INPUTTEXT" == "" ]; then
      			INPUTTEXT=1
    		fi


    		# If the value is 0 here, silently change the value of OSSEC_NOTIFY to "no".
    		if [ $INPUTTEXT -eq 0 ]; then
      			export OSSEC_NOTIFY="no"
      			INPUTTEXT=1
    		fi

    		if [ "$INPUTTEXT" == "" ]; then
      			INPUTTEXT=1
    		fi

    		# Update the maximum number of messages per hour sent by OSSEC in the
    		# ASL config file.
    		perl -pi -e "s/^OSSEC_MAX_MSG=.*/OSSEC_MAX_MSG=\"$INPUTTEXT\"/" /etc/asl/config

    		# A little slight-of-code to edit the ASL config file in the event that
    		# OSSEC isn't supposed to send notifications that will survive later
    		# reconfiguration runs (and not break OSSEC, which should know better).
    		if [ "$OSSEC_NOTIFY" == "no" ]; then
			perl -pi -e "s/^OSSEC_NOTIFY=.*/OSSEC_NOTIFY=\"no\"/" /etc/asl/config
    		fi
		perl -pi -e "s/^HIDS_SHUN_TRACKING=.*/HIDS_SHUN_TRACKING=\"yes\"/" /etc/asl/config

	else


  		echo
		echo "ASL client mode configuration"
		echo

		# if the server is installed, remove it.
		if rpm --quiet -q ossec-hids-server ; then
			echo -n "  Server install detected. Removing server package: "
			yum -y remove ossec-hids-server > /dev/null
			RETVAL=$?
			if [  $RETVAL -ge 1 ]; then
				echo "Failed."
				echo "  Error: ASL server could not be removed. Configuration halted."
				exit 1
			else
				echo "Complete"
			fi
		fi

		# if asl-web is installed, remove it and tortixd
		if rpm --quiet -q asl-web ; then
			echo -n "  Web Console install detected. Removing console package: "
			yum -y remove tortixd asl-web > /dev/null
			RETVAL=$?
			if [  $RETVAL -ge 1 ]; then
				echo "Failed."
				echo "  Error: ASL Web Console could not be removed. Configuration halted."
				exit 1
			else
				echo "Complete"
			fi
		fi


		if ! rpm --quiet -q ossec-hids-client; then
			echo -n "  Installing ASL client: "
			/etc/init.d/psmon stop >/dev/null 2>&1
			yum -y install ossec-client > /dev/null 
			RETVAL=$?
				if [  $RETVAL -ge 1 ]; then
				echo "Failed."
				echo "  Error: ASL client could not be installed. Configuration halted."
				exit 1
			else
				echo "Complete."
			fi
      			/etc/init.d/psmon start >/dev/null 2>&1
		fi



		# TODO: option to manually import key or to use authd
    		# Configure the OSSEC client with the server it's supposed to
		    # report to.
		if [ ! $AUTO ] && [ $LK_MODE -lt 1 ]; then
    			check_input "  Enter Address or hostname to the ASL Console server:" \
				    "^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$" ""
    			OSSEC_SERVER=$INPUTTEXT
    
	    		echo "  Provide the key generated by the server."
    			echo "  The best approach is to cut and paste it."
    			echo -n "  Paste it here (ctrl-c to quit): "
    			read KEY < $SSH_TTY

		fi
    		# Set the agent key.
    		echo y | /var/ossec/bin/manage_client -i $KEY >/dev/null 2>&1

    		if [ -f /var/ossec/etc/client.keys ]; then
      			echo "  Key Added."
      			echo
    		else
      			echo "  Error: Invalid key."
      			echo
    		fi

    		perl -pi -e "s/^OSSEC_SERVER=.*/OSSEC_SERVER=\"$OSSEC_SERVER\"/" /etc/asl/config
		perl -pi -e "s/^HIDS_SHUN_TRACKING=.*/HIDS_SHUN_TRACKING=\"no\"/" /etc/asl/config


	fi # End client server logic


	# Build the list of users permitted to SSH into the system.
	echo
	echo "Administration"
	echo "  Administrative users (other than root)  will be permitted to SSH into "
	echo "  the system, and use privileged functions. Defining administrative users"
	echo "  will disable root logins, and disable password based authentication, if"
	echo "  those users have installed SSH keys."
	echo 
	if [ ! $AUTO ]; then
		echo -n "  Administrators (separated by whitespace) [Default: $ADMIN_USERS]: "
		read A_USERS < $SSH_TTY
	fi
  
	if [ "$A_USERS" != "" ]; then
		ADMIN_USERS=$A_USERS
  	fi

	# lint check for root.
	for userid in $A_USERS; do
		if [ $userid != "root" ]; then
			ADMIN_USERS="$ADMIN_USERS $userid"
		else
			echo "
  $userid is not a valid Administrative user. Administrative 
  users are non root users permitted access to the root user account.
  Excluding the root user from the Administrator list automatically."
			echo 
		fi
	done
  

  # Build a list of whitelisted IP addresses.
  echo 
  echo "Active Response and Whitelisting"
  echo "  ASL has an active countermeasures system, attacks detected by ASL will be"
  echo "  blocked inline, and by firewall rules. "
  echo "  Whitelisted IP's will not be blocked by ASL active countermeasures. "
  echo "  Only use whitelists for monitoring, administrative, or key systems that you "
  echo "  always want to allow."
  echo
  # Enable or disable active response in OSSEC.
  if [ $LK_MODE -ge 1 ]; then 
  	OSSEC_ACTIVE_RESPONSE="yes"
  elif [ ! $AUTO ]; then
  	check_input "  Enable ASL active response to attacks? (yes/no) [Default: $OSSEC_ACTIVE_RESPONSE ]:" "yes|no" $OSSEC_ACTIVE_RESPONSE
  	OSSEC_ACTIVE_RESPONSE=$INPUTTEXT
  fi
  perl -pi -e "s/^OSSEC_ACTIVE_RESPONSE=.*/OSSEC_ACTIVE_RESPONSE=\"$OSSEC_ACTIVE_RESPONSE\"/" /etc/asl/config

  if [ ! $AUTO ] && [ $LK_MODE -lt 1 ]; then

	if [ -f /etc/resolv.conf ]; then
		awk '/^nameserver/ {print $2}' /etc/resolv.conf  >> /etc/asl/whitelist

	fi
	
	if [ -f /etc/asl/whitelist ]; then
		echo
		echo "  Current Whitelist"
		for i in `cat /etc/asl/whitelist`; do
			echo "    $i"
		done
		echo
		echo
	fi
  	echo -n "  IP Whitelist (separated by whitespace) [Default: none]: "
  	read IP_WHITELIST < $SSH_TTY
  fi
  IP_WHITELIST="$IP_WHITELIST"

  # add local IP's
  for i in `/sbin/ip addr | grep -v 'inet6' | grep -v '127\.[0-9]*\.[0-9]*\.[0-9]*' | sed -n -e 's/^ *inet \([0-9\.]*\).*$/\1/ p'`; do
    IP_WHITELIST="$IP_WHITELIST  $i"
  done

  # add client IP
  LOCAL_IP=$(echo $SSH_CLIENT | awk '{print $1}')
  if [ $LOCAL_IP ]; then
	INPUTTEXT=yes
	if [ ! $AUTO  ] && [ $LK_MODE -lt 1 ]; then
  		check_input "  Add your current IP ($LOCAL_IP) to whitelist? [Default: yes] (yes/no)" "yes|no" yes
	fi
  	if [ $INPUTTEXT = "yes" ]; then
		echo $LOCAL_IP >> /etc/asl/whitelist
  	fi
  fi




  # Commit changes
  perl -pi -e "s/^EMAIL=.*/EMAIL=\"`make_regex $EMAIL`\"/"  /etc/asl/config 
  perl -pi -e "s/^ADMIN_USERS=.*/ADMIN_USERS=\"$ADMIN_USERS\"/"  /etc/asl/config 


  for ip_addr in $IP_WHITELIST; do
    echo $ip_addr >> /etc/asl/whitelist
  done
  
  sort -u /etc/asl/whitelist > /etc/asl/whitelist.tmp
  mv /etc/asl/whitelist.tmp /etc/asl/whitelist

}

function kernelsettings {
    # Disable the addition of LKMs?
    echo
    echo 
    echo "Kernel Settings"
    echo "  ASL allows you to disable kernel module loading after the system "
    echo "  has booted. This prevents an intruder from loading kernel modules, "
    echo "  such as LKM (Loadable Kernel Module) rootkits, into the running system."
    echo "  Loading kernel modules is disabled by default."
    echo 

    if [ ! $AUTO ]; then
    	check_input "  Allow run-time kernel module loading? (yes/no) [Default: $ALLOW_kmod_loading ]:" "yes|no" $ALLOW_kmod_loading
    	ALLOW_kmod_loading=$INPUTTEXT
    fi

    perl -pi -e "s/^ALLOW_kmod_loading=.*/ALLOW_kmod_loading=\"$ALLOW_kmod_loading\"/"  /etc/asl/config 

	
    

  # Plesk is not compatible with HARDEN_PTRACE	
  if rpm --quiet -q psa ; then
    perl -p -i -e "s[HARDEN_PTRACE=.*][HARDEN_PTRACE=\"no\"]g" /etc/asl/config
    perl -p -i -e "s[FW_PLESK_UPDATES=.*][FW_PLESK_UPDATES=\"yes\"]g" /etc/asl/config
  fi

}
# firewall_rule_check <msg> <var>
# returns variable
function firewall_rule_check {
	msg=$1
	var=$2


	failed=0
	CONFIRMED=0
	while [ $CONFIRMED -lt 1 ]; do
		if [ $failed -gt 2 ]; then
			echo
			echo "Exiting: too many failed entries." |tee -a $LOG
			exit 1

		fi

		var=$(echo $var | sed "s/,/ /g")
		var=$(for i in $var; do echo $i; done | sort -un)
		var=$(echo $var| sed "s/ /,/g")


		echo -n "  $msg [Default: $var]: "
		read INPUTTEXT < $SSH_TTY
		if [ "$INPUTTEXT" != "" ]; then

			# enumerate size of array
			i_count=0
			for i in $(echo $INPUTTEXT | sed 's/,/ /g'); do
				i_count=$(( $i_count +1 ))
				
			done

			y_count=0
			for i in $(echo $INPUTTEXT | sed 's/,/ /g'); do
		
				y_count=$(( $y_count + 1))
				echo $i|egrep '^[0-9]+$' >/dev/null
				if [ $? -ne 0 ]; then
					failed=$(( $failed + 1 ))
					echo "	Error: ($i) is not a number between (1-65535)"
					break
				elif [ $i -gt 65535 2>/dev/null ]; then
					echo "	Error: ($i) is greater than (65535)"
					failed=$(( $failed + 1 ))
					break
				fi
				if [ $i_count -eq $y_count  ]; then
					CONFIRMED=1
				fi
			done
		else
			CONFIRMED=1
			INPUTTEXT=$var
			
		fi

	done
			
	return
}

function firewall_policy {
  	echo
  	echo
  	echo "Firewall Policy Settings"
	echo "  The ASL firewall consists of multiple advanced firewall capabilities"
  	echo "  * Inbound TCP connections"
  	echo "  * Inbound UDP connections"
  	echo "  * Outbound TCP connections"
  	echo "  * Outbound UDP connections"
  	echo "  * Restricted SMTP Outbound connections by userid"
  	echo "  * Dynamic Anti-Spam, and Anti-Malware blacklists"
  	echo "  * Blocking Geographic regions"
	echo


  	check_input "  Configure ASL Console Access List (yes/no) [Default: yes] ?" "yes|no" yes 
	if [ "$INPUTTEXT" == "yes" ]; then
		if [ -f /etc/asl/firewall/tortixd-access-list ]; then
			echo
			echo "  Current Access list"
			for i in `cat /etc/asl/firewall/tortixd-access-list`; do
				echo "    $i"
			done
			echo
			echo
		fi
                IPCONFIRMED=0
		regex="\b(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b"
		while [ $IPCONFIRMED -lt 1 ]; do
			echo -n "    Enter List of IPs to allow access to the ASL Console [Enter to exit]: "

		        read ipaddr < $SSH_TTY
        		if [[ "$ipaddr" == "" ]]
        		then
                		IPCONFIRMED=1
       			else

			CHECK=$(echo $ipaddr | egrep $regex)
			if [[ "$?" -eq 0 ]]
			then
				echo $ipaddr >> /etc/asl/firewall/tortixd-access-list
			else
				echo -n "    Incorrect IP address, please try again [Enter to exit]: "
			fi
		fi

done
	fi
	if [ ! -f /etc/asl/firewall/tortixd-access-list ]; then
		# Set console to open
		echo "0.0.0.0/0" >> /etc/asl/firewall/tortixd-access-list
	fi
	

  	check_input "  Configure Inbound TCP Firewall policy (yes/no) [Default: yes] ?" "yes|no" yes 
	if [ "$INPUTTEXT" == "no" ]; then
		return
	fi
	
	echo
	echo "  Services added during this step will be INBOUND TCP services policy"
	echo
        echo "  The following external TCP services were detected:"

	if [ "$FW_INBOUND_TCP_SERVICES" == "no" ]; then
		FW_INBOUND_TCP_SERVICES=""
	fi
	FW_INBOUND_TCP_SERVICES="$BASE_PORTS,$FW_INBOUND_TCP_SERVICES"
	# Existing TCP rules
      	SERVICES=$(netstat -lpn | grep tcp |sed 's/:::/0.0.0.0:/g' | awk '{print $4 ":" $7}' | sort -t: -k2 -n |uniq)

        for i in $SERVICES ; do


		ip=$(echo $i | awk -F: '{print $1}')
		port=$(echo $i | awk -F: '{print $2}')
		program=$(echo $i | awk -F/ '{print $2}')

		if [ "$ip" != "127.0.0.1" ]; then
			echo "	TCP Port: $port listens on $ip ($program)"
			if [ "$FW_INBOUND_TCP_SERVICES" == "" ]; then
				FW_INBOUND_TCP_SERVICES="$port"
			else
				FW_INBOUND_TCP_SERVICES="$FW_INBOUND_TCP_SERVICES,$port"
			fi
		fi

	done
	echo


	firewall_rule_check "Inbound TCP firewall policy" $FW_INBOUND_TCP_SERVICES
	if [ $? -ne 1 ] ; then
		FW_INBOUND_TCP_SERVICES=$INPUTTEXT
	fi

	echo
	echo "  ASL can configure Outbound TCP services that are allowed for the system "
	echo "  Services added during this step will be the OUTBOUND TCP services policy"
	echo "  selecting no at this step will disable outbound service filtering."
	echo

  	check_input "  Configure Firewall Outbound policy (yes/no) [Default: yes] ?" "yes|no" yes 
	if [ "$INPUTTEXT" != "no" ]; then
		firewall_rule_check "Outbound TCP firewall policy" $FW_OUTPUT_TCP_SERVICES

		if [ $? -ne 1 ] && [ "$INPUTTEXT" != "" ] ; then
			FW_OUTPUT_TCP_SERVICES=$INPUTTEXT
		else
			FW_OUTPUT_TCP_SERVICES=no
		fi
	fi


	# Inbound UDP
	echo
	echo "  ASL can restrict Inbound UDP traffic to specified ports"
	echo
  	check_input "  Configure Inbound UDP Firewall policy (yes/no) [Default: yes] ?" "yes|no" yes 
	if [ "$INPUTTEXT" == "no" ]; then
		return
	fi

	
	echo
	echo "  Services added during this step will be INBOUND UDP services policy"
	echo
        echo "  The following external UDP services were detected:"
	if [ "$FW_INBOUND_UDP_SERVICES" == "no" ]; then
		FW_INBOUND_UDP_SERVICES=""
	fi

	for i in $(netstat -lpn | egrep "udp.*[0-9]{1,3}\.[0-9]{1,3}\.([0-9]{1,3})" |awk '{print $4 ":" $6}' |sort -t: -k2 -n ) ; do
		ip=$(echo $i | awk -F: '{print $1}')
		port=$(echo $i | awk -F: '{print $2}')
		program=$(echo $i | awk -F/ '{print $2}')

		if [ "$ip" != "127.0.0.1" ]; then
			echo "	UDP Port: $port listens on $ip ($program)"
			if [ "$FW_INBOUND_UDP_SERVICES" == "" ]; then
				FW_INBOUND_UDP_SERVICES="$port"
			else
				FW_INBOUND_UDP_SERVICES="$FW_INBOUND_UDP_SERVICES,$port"
			fi
		fi

	done
	echo

	firewall_rule_check "Inbound UDP firewall policy" $FW_INBOUND_UDP_SERVICES
        if [ $? -ne 1 ] ; then
                FW_INBOUND_UDP_SERVICES=$INPUTTEXT
        fi

	# Outbound UDP
	echo
	echo "  ASL can configure Outbound UDP services that are allowed for the system "
	echo "  Services added during this step will be the OUTBOUND UDP services policy"
	echo "  selecting no at this step will disable outbound service filtering."
	echo

  	check_input "  Configure Firewall Outbound UDP policy (yes/no) [Default: yes] ?" "yes|no" yes 
	if [ "$INPUTTEXT" != "no" ]; then
		firewall_rule_check "Outbound UDP firewall policy" $FW_OUTPUT_UDP_SERVICES
		if [ $? -ne 1 ] && [ "$FW_OUTPUT_UDP_SERVICES" != "" ] ; then
			FW_OUTPUT_UDP_SERVICES=$INPUTTEXT
		else
			FW_OUTPUT_UDP_SERVICES=no
		fi
	fi

	echo
	echo "  Dynamic Blacklist Configuration"
	echo "  ASL blacklists are applied to both INPUT and OUTPUT filters."
	echo "  All blacklists log by default if enabled"
	echo
	echo "	* AutoShun: AutoShun is a community sourced blacklist from the Snort IDS." 
	echo "	* C.I. Army: Collective Intelligence blacklist is a community blacklist of known malicious actors." 
	echo "	* Dshield: A community based firewall log correlation system."
	echo "	* Emerging Threats: Russian Business Network (RBN) blacklist" 
	echo "	* Lasso: Spamhaus DROP list for known spam controlled address space." 
	echo "	* ELasso: Extended Spamhaus DROP list for known spam controlled address space." 
	echo "	* OpenBL:  OpenBL tracks multiple service abuse, including ssh, ftp, smtp, and http." 
	echo "	* OpenProxies: Known Open Proxy server blacklist." 
	echo "	* TOR: The Onion Router exit node list." 
	echo 

	# autoshun
  	check_input "  Enable AutoShun blacklist (yes/no) [Default: $FW_AUTOSHUN] ?" "yes|no" $FW_AUTOSHUN
  	FW_AUTOSHUN=$INPUTTEXT
	FW_AUTOSHUN_LOG=$INPUTTEXT

	# c.i. army
  	check_input "  Enable C.I. Army blacklist (yes/no) [Default: $FW_CIARMY] ?" "yes|no" $FW_CIARMY
  	FW_CIARMY=$INPUTTEXT
	FW_CIARMY_LOG=$INPUTTEXT

	# Dshield
  	check_input "  Enable Dshield blacklist (yes/no) [Default: $FW_DSHIELD] ?" "yes|no" $FW_DSHIELD
  	FW_DSHIELD=$INPUTTEXT
	FW_DSHIELD_LOG=$INPUTTEXT

	# emerging threats
  	check_input "  Enable Emerging Threats blacklist (yes/no) [Default: $FW_EMERGING_THREATS] ?" "yes|no" $FW_EMERGING_THREATS
  	FW_EMERGING_THREATS=$INPUTTEXT
	FW_EMERGING_THREATS_LOG=$INPUTTEXT

	# Lasso
  	check_input "  Enable Spamhaus LASSO blacklist (yes/no) [Default: $FW_LASSO] ?" "yes|no" $FW_LASSO
  	FW_LASSO=$INPUTTEXT
	FW_LASSO_LOG=$INPUTTEXT

	# ELasso
  	check_input "  Enable Spamhaus Extended LASSO blacklist (yes/no) [Default: $FW_ELASSO] ?" "yes|no" $FW_ELASSO
  	FW_ELASSO=$INPUTTEXT
	FW_ELASSO_LOG=$INPUTTEXT

	# OpenBL
  	check_input "  Enable OpenBL blacklist (yes/no) [Default: $FW_OPENBL] ?" "yes|no" $FW_OPENBL
  	FW_OPENBL=$INPUTTEXT
	FW_OPENBL_LOG=$INPUTTEXT

	# OpenProxy
  	check_input "  Enable Open Proxies blacklist (yes/no) [Default: $FW_OPENPROXIES] ?" "yes|no" $FW_OPENPROXIES
  	FW_OPENPROXIES=$INPUTTEXT
	FW_OPENPROXIES_LOG=$INPUTTEXT

	# TOR
  	check_input "  Enable TOR blacklist (yes/no) [Default: $FW_TOR] ?" "yes|no" $FW_TOR
  	FW_TOR=$INPUTTEXT
	FW_TOR_LOG=$INPUTTEXT

	# TODO: OUTPUT_MTA
	# TODO: FW_PORTSCAN
	# TODO: FW_LOWLEVEL_PORTSCAN
	# TODO: Plesk
	# TODO: SPAMASSASSIN

	# disable iptables from startup

	if [ "$DIST" == "el7" ]; then
		/usr/bin/systemctl disable firewalld 
	else
		/sbin/chkconfig iptables off
	fi

    	perl -pi -e "s/^FW_INBOUND_TCP_SERVICES=.*/FW_INBOUND_TCP_SERVICES=\"$FW_INBOUND_TCP_SERVICES\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_INBOUND_UDP_SERVICES=.*/FW_INBOUND_UDP_SERVICES=\"$FW_INBOUND_UDP_SERVICES\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_OUTPUT_TCP_SERVICES=.*/FW_OUTPUT_TCP_SERVICES=\"$FW_OUTPUT_TCP_SERVICES\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_OUTPUT_UDP_SERVICES=.*/FW_OUTPUT_UDP_SERVICES=\"$FW_OUTPUT_UDP_SERVICES\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_AUTOSHUN=.*/FW_AUTOSHUN=\"$FW_AUTOSHUN\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_AUTOSHUN_LOG=.*/FW_AUTOSHUN_LOG=\"$FW_AUTOSHUN_LOG\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_DSHIELD=.*/FW_DSHIELD=\"$FW_DSHIELD\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_DSHIELD_LOG=.*/FW_DSHIELD_LOG=\"$FW_DSHIELD_LOG\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_LASSO=.*/FW_LASSO=\"$FW_LASSO\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_LASSO_LOG=.*/FW_LASSO_LOG=\"$FW_LASSO_LOG\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_ELASSO=.*/FW_ELASSO=\"$FW_ELASSO\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_ELASSO_LOG=.*/FW_ELASSO_LOG=\"$FW_ELASSO_LOG\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_CIARMY=.*/FW_CIARMY=\"$FW_CIARMY\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_CIARMY_LOG=.*/FW_CIARMY_LOG=\"$FW_CIARMY_LOG\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_EMERGING_THREATS=.*/FW_EMERGING_THREATS=\"$FW_EMERGING_THREATS\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_EMERGING_THREATS_LOG=.*/FW_EMERGING_THREATS_LOG=\"$FW_EMERGING_THREATS_LOG\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_OPENBL=.*/FW_OPENBL=\"$FW_OPENBL\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_OPENBL_LOG=.*/FW_OPENBL_LOG=\"$FW_OPENBL_LOG\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_OPENPROXIES=.*/FW_OPENPROXIES=\"$FW_OPENPROXIES\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_OPENPROXIES_LOG=.*/FW_OPENPROXIES_LOG=\"$FW_OPENPROXIES_LOG\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_TOR=.*/FW_TOR=\"$FW_TOR\"/"  /etc/asl/config 
    	perl -pi -e "s/^FW_TOR_LOG=.*/FW_TOR_LOG=\"$FW_TOR_LOG\"/"  /etc/asl/config 


	# Spamassassin fw policy
	if rpm --quiet -q spamassassin ; then
		perl -p -i -e "s[FW_SPAMASSASSIN_UPDATES=.*][FW_SPAMASSASSIN_UPDATES=\"yes\"]g" /etc/asl/config
	fi

	
}



function php_settings {
  # PHP Functions
  echo
  echo
  echo "PHP Settings"
  echo "  ASL can check php configuration settings for high risk functions,"
  echo "  and other configuration settings."
  echo

  if [ -f /etc/php.ini ]; then
	PHP_INI=/etc/php.ini
  elif [ -f /usr/local/lib/php.ini ]; then
	PHP_INI=/usr/local/lib/php.ini
  fi


  if [ ! $AUTO  ]; then
  	check_input "  Enable PHP checks (yes/no) [Default: $PHP_CHECKS] ?" "yes|no" $PHP_CHECKS
  	PHP_CHECKS=$INPUTTEXT
  fi


  if [ "$PHP_CHECKS" == "no" ]; then
    	perl -pi -e "s/^PHP_CHECKS=.*/PHP_CHECKS=\"$PHP_CHECKS\"/"  /etc/asl/config 
	perl -pi -e "s[^disable_functions.*][disable_functions = ]g" $PHP_INI
    	return
  fi


  if [ ! $AUTO ]; then
	  # PHP_SAFE_MODE=on
	  check_input "  Enable PHP Safe Mode [Default: $PHP_SAFE_MODE]? (yes/no)" "yes|no" $PHP_SAFE_MODE
	  PHP_SAFE_MODE=$INPUTTEXT

	  # PHP_REGISTER_GLOBALS
	  check_input "  Allow register_globals [Default: $PHP_REGISTER_GLOBALS]? (yes/no)" "yes|no"  $PHP_REGISTER_GLOBALS
	  PHP_REGISTER_GLOBALS=$INPUTTEXT
  fi

  # FIXUP, detect horde
  DEFAULT_ALLOW="psa-horde squirrelmail psa-atmail roundcubemail"
  DEFAULTYN1=no
  DEFAULTYN2=no
  DEFAULTYN3=no
  DEFAULTYN4=no
  DEFAULTYN5=no
  HORDE=0

  for check in $DEFAULT_ALLOW; do
    if rpm --quiet -q $check; then
	HORDE=1
	break
    fi
  done

  if [ -d /usr/local/cpanel ]; then
    HORDE=1
  fi

  # Horde
  if rpm --quiet -q psa-horde; then
        echo "  ** Horde Webmail detected fsockopen, escapeshellcmd are required **"
	DEFAULTYN3=yes
	DEFAULTYN4=yes
  fi

  if [ $HORDE -gt 0 ];then
    echo "  ** Atmail, Roundcube or Squirrelmail detected, exec, popen, fsockopen, escapeshellcmd are required **"
    DEFAULTYN1=yes
    DEFAULTYN2=yes
    DEFAULTYN3=yes
    DEFAULTYN4=yes
    DEFAULTYN5=yes
  fi

  if [ ! $AUTO  ]; then
	  check_input "  Allow function exec (Risk: High) [Default: $DEFAULTYN]? (yes/no)" "yes|no" $DEFAULTYN1
	  ALLOW_exec=$INPUTTEXT

	  #ALLOW_popen=no
	  check_input "  Allow function popen (Risk: High) [Default: $DEFAULTYN]? (yes/no)" "yes|no" $DEFAULTYN2
	  ALLOW_popen=$INPUTTEXT

	  check_input "  Allow function fsockopen (Risk: High) [Default: $DEFAULTYN]? (yes/no)" "yes|no" $DEFAULTYN3
	  ALLOW_fsockopen=$INPUTTEXT

	  #ALLOW_escapeshellcmd=no
	  check_input "  Allow function escapeshellcmd (Risk: Low) [Default: $DEFAULTYN]? (yes/no)" "yes|no" $DEFAULTYN4
	  ALLOW_escapeshellcmd=$INPUTTEXT

	  #ALLOW_pfsockopen=no
	  check_input "  Allow function pfsockopen (Risk: High) [Default: $DEFAULTYN]? (yes/no)" "yes|no" $DEFAULTYN5
	  ALLOW_pfsockopen=$INPUTTEXT

	  #ALLOW_shell_exec=no
	  check_input "  Allow function shell_exec (Risk: High) [Default: $ALLOW_shell_exec]? (yes/no)" "yes|no" $ALLOW_shell_exec
	  ALLOW_shell_exec=$INPUTTEXT

	  #ALLOW_system=no
	  check_input "  Allow function system (Risk: High) [Default: $ALLOW_system]? (yes/no)" "yes|no" $ALLOW_system
	  ALLOW_system=$INPUTTEXT

	  #ALLOW_curl_exec=no
	  check_input "  Allow function curl_exec (Risk: High) [Default: $ALLOW_curl_exec]? (yes/no)" "yes|no" $ALLOW_curl_exec
	  ALLOW_curl_exec=$INPUTTEXT

	  #ALLOW_curl_multi_exec=no
	  check_input "  Allow function curl_multi_exec (Risk: High) [Default: $ALLOW_curl_multi_exec]? (yes/no)" "yes|no" $ALLOW_curl_multi_exec
	  ALLOW_curl_multi_exec=$INPUTTEXT

	  #ALLOW_pcntl_exec=no
	  check_input "  Allow function pcntl_exec (Risk: High) [Default: $ALLOW_pcntl_exec]? (yes/no)" "yes|no" $ALLOW_pcntl_exec
	  ALLOW_pcntl_exec=$INPUTTEXT

	  #ALLOW_ftp_exec=no
	  check_input "  Allow function ftp_exec (Risk: Moderate) [Default: $ALLOW_ftp_exec]? (yes/no)" "yes|no" $ALLOW_ftp_exec
	  ALLOW_ftp_exec=$INPUTTEXT

	  #ALLOW_dl=no
	  check_input "  Allow function dl (Risk: High) [Default: $ALLOW_dl]? (yes/no)" "yes|no" $ALLOW_dl
	  ALLOW_dl=$INPUTTEXT
	 
	  #ALLOW_leak=no
	  check_input "  Allow function leak (Risk: Moderate) [Default: $ALLOW_leak]? (yes/no)" "yes|no" $ALLOW_leak
	  ALLOW_leak=$INPUTTEXT

	  #ALLOW_passthru=no
	  check_input "  Allow function passthru (Risk: High) [Default: $ALLOW_passthru]? (yes/no)" "yes|no" $ALLOW_passthru
	  ALLOW_passthru=$INPUTTEXT


	  #ALLOW_phpinfo=yes
	  check_input "  Allow function phpinfo (Risk: Low) [Default: $ALLOW_phpinfo]? (yes/no)" "yes|no" $ALLOW_phpinfo
	  ALLOW_phpinfo=$INPUTTEXT


	  #ALLOW_posix_kill=no
	  check_input "  Allow function posix_kill (Risk: High) [Default: $ALLOW_posix_kill]? (yes/no)" "yes|no" $ALLOW_posix_kill
	  ALLOW_posix_kill=$INPUTTEXT

	  #ALLOW_posix_mkfifo=no
	  check_input "  Allow function posix_mkfifo (Risk: High) [Default: $ALLOW_posix_mkfifo]? (yes/no)" "yes|no" $ALLOW_posix_mkfifo
	  ALLOW_posix_mkfifo=$INPUTTEXT

	  #ALLOW_posix_setpgid=no
	  check_input "  Allow function posix_setpgid (Risk: Moderate) [Default: $ALLOW_posix_setpgid]? (yes/no)" "yes|no" $ALLOW_posix_setpgid
	  ALLOW_posix_setpgid=$INPUTTEXT

	  #ALLOW_posix_setsid=no
	  check_input "  Allow function posix_setsid (Risk: Moderate) [Default: $ALLOW_posix_setsid]? (yes/no)" "yes|no" $ALLOW_posix_setsid
	  ALLOW_posix_setsid=$INPUTTEXT

	  #ALLOW_posix_setuid=no
	  check_input "  Allow function posix_setuid (Risk: High) [Default: $ALLOW_posix_setuid]? (yes/no)" "yes|no" $ALLOW_posix_setuid
	  ALLOW_posix_setuid=$INPUTTEXT

	  #ALLOW_proc_close=no
	  check_input "  Allow function proc_close (Risk: High) [Default: $ALLOW_proc_close]? (yes/no)" "yes|no" $ALLOW_proc_close
	  ALLOW_proc_close=$INPUTTEXT

	  #ALLOW_proc_get_status=no
	  check_input "  Allow function status (Risk: Moderate) [Default: $ALLOW_proc_get_status]? (yes/no)" "yes|no" $ALLOW_proc_get_status
	  ALLOW_proc_get_status=$INPUTTEXT

	  #ALLOW_proc_nice=no
	  check_input "  Allow function proc_nice (Risk: Moderate) [Default: $ALLOW_proc_nice]? (yes/no)" "yes|no" $ALLOW_proc_nice
	  ALLOW_proc_nice=$INPUTTEXT

	  #ALLOW_proc_open=no
	  check_input "  Allow function proc_open (Risk: High) [Default: $ALLOW_proc_open]? (yes/no)" "yes|no" $ALLOW_proc_open
	  ALLOW_proc_open=$INPUTTEXT

	  #ALLOW_proc_terminate=no
	  check_input "  Allow function proc_terminate (Risk: High) [Default: $ALLOW_proc_terminate]? (yes/no)" "yes|no" $ALLOW_proc_terminate
	  ALLOW_proc_terminate=$INPUTTEXT

	  #ALLOW_show_source=no
	  check_input "  Allow function show_source (Risk: Moderate) [Default: $ALLOW_show_source]? (yes/no)" "yes|no" $ALLOW_show_source
  	  ALLOW_show_source=$INPUTTEXT
  fi

  # Update config
  perl -pi -e "s/^PHP_CHECKS=.*/PHP_CHECKS=\"$PHP_CHECKS\"/"  /etc/asl/config
  perl -pi -e "s/^PHP_SAFE_MODE=.*/PHP_SAFE_MODE=\"$PHP_SAFE_MODE\"/" /etc/asl/config
  perl -pi -e "s/^PHP_REGISTER_GLOBALS=.*/PHP_REGISTER_GLOBALS=\"$PHP_REGISTER_GLOBALS\"/" /etc/asl/config
  perl -pi -e "s/^ALLOW_dl=.*/ALLOW_dl=\"$ALLOW_dl\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_exec=.*/ALLOW_exec=\"$ALLOW_exec\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_leak=.*/ALLOW_leak=\"$ALLOW_leak\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_passthru=.*/ALLOW_passthru=\"$ALLOW_passthru\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_pfsockopen=.*/ALLOW_pfsockopen=\"$ALLOW_pfsockopen\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_popen=.*/ALLOW_popen=\"$ALLOW_popen\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_escapeshellcmd=.*/ALLOW_escapeshellcmd=\"$ALLOW_escapeshellcmd\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_fsockopen=.*/ALLOW_fsockopen=\"$ALLOW_fsockopen\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_posix_kill=.*/ALLOW_posix_kill=\"$ALLOW_posix_kill\"/" \
	/etc/asl/config
  perl -pi -e "s/^ALLOW_posix_mkfifo=.*/ALLOW_posix_mkfifo=\"$ALLOW_posix_mkfifo\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_posix_setpgid=.*/ALLOW_posix_setpgid=\"$ALLOW_posix_setpgid\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_posix_setsid=.*/ALLOW_posix_setsid=\"$ALLOW_posix_setsid\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_posix_setuid=.*/ALLOW_posix_setuid=\"$ALLOW_posix_setuid\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_proc_close=.*/ALLOW_proc_close=\"$ALLOW_proc_close\"/" \
	/etc/asl/config
  perl -pi -e "s/^ALLOW_proc_get_status=.*/ALLOW_proc_get_status=\"$ALLOW_proc_get_status\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_proc_nice=.*/ALLOW_proc_nice=\"$ALLOW_proc_nice\"/" \
	/etc/asl/config
  perl -pi -e "s/^ALLOW_proc_open=.*/ALLOW_proc_open=\"$ALLOW_proc_open\"/" \
	/etc/asl/config
  perl -pi -e "s/^ALLOW_proc_terminate=.*/ALLOW_proc_terminate=\"$ALLOW_proc_terminate\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_shell_exec=.*/ALLOW_shell_exec=\"$ALLOW_shell_exec\"/" \
	/etc/asl/config
  perl -pi -e "s/^ALLOW_show_source=.*/ALLOW_show_source=\"$ALLOW_show_source\"/" \
	/etc/asl/config
  perl -pi -e "s/^ALLOW_system=.*/ALLOW_system=\"$ALLOW_system\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_curl_exec=.*/ALLOW_curl_exec=\"$ALLOW_curl_exec\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_curl_multi_exec=.*/ALLOW_curl_multi_exec=\"$ALLOW_curl_multi_exec\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_pcntl_exec=.*/ALLOW_pcntl_exec=\"$ALLOW_pcntl_exec\"/"  /etc/asl/config
  perl -pi -e "s/^ALLOW_ftp_exec=.*/ALLOW_ftp_exec=\"$ALLOW_ftp_exec\"/"  /etc/asl/config
}

# This function is used to detect special environments, like cpanel
function environment_detect {

  get_distro
  PLATFORM="standard"
  BASE_PORTS=""

  if [ "$DIST" != "el5" ] && [ "$DIST" != "el6" ] && [ "$DIST" != "el7" ]; then
	perl -pi -e "s[MODSEC_RULES_PATH.*][MODSEC_RULES_PATH=\"/etc/apache2/modsecurity.d\"]" /etc/asl/config 
	if [ ! -d /etc/apache2/modsecurity.d ]; then
		mkdir -p /etc/apache2/modsecurity.d
	fi

	if [ -d /etc/apache2/conf-enabled ]; then
		ln -sf /etc/apache2/conf-enabled /etc/apache2/conf.d
	fi

	if [  -f /usr/sbin/apache2 ]; then
		ln -sf /usr/sbin/apache2 /usr/sbin/httpd
	fi
	
  fi
  if [ -d /usr/local/psa ]; then
    BASE_PORTS="21,22,25,53,80,106,110,143,443,465,587,990,993,995,3306,5432,8443,8447,8880,9080"
  fi

  if [ -d /usr/local/cpanel ]; then
    echo "  Detected Cpanel environment...."
    perl -pi -e "s/SYSTEM_TYPE.*/SYSTEM_TYPE=\"cpanel\"/" /etc/asl/config 
    PLATFORM="cpanel"
    BASE_PORTS="21,22,25,26,53,80,110,143,443,465,873,993,995,2078,2082,2083,2086,2087,2089,2085,2096,3306,6666"
  fi

  if [ -d /usr/local/directadmin ]; then
    echo "  Detected DirectAdmin environment...."
    perl -pi -e "s/SYSTEM_TYPE.*/SYSTEM_TYPE=\"directadmin\"/" /etc/asl/config 
    PLATFORM="directadmin"
    BASE_PORTS="21,22,25,53,80,110,143,443,587,993,995,2222,3306"
  fi

  if [ "$DIST" == "el5" -o "$DIST" == "el6" -o "$DIST" == "el7" ]; then
  	if ! rpm -q psmon >/dev/null; then
 		perl -pi -e "s/PSMON_ENABLED=.*/PSMON_ENABLED=\"no\"/" /etc/asl/config 
  	fi
  fi

  if [ $LK_MODE -ge 1 ]; then
  	ASL=0
  else
	ASL=0
  	if [ "$DIST" != "el5" ] && [ "$DIST" != "el6" ] && [ "$DIST" != "el7" ]; then
		ASL=0
	else
          if  ! rpm -q asl > /dev/null ; then
		echo
		echo
		check_input "  ASL not detected, would you like to upgrade to ASL now? [Default: yes]:" "yes|no" yes
		if [ "$INPUTTEXT" == "yes" ]; then
			ASL=1
			/var/asl/lib/modules/asl-installer
		fi
		
	  else
		ASL=1
	 	perl -pi -e "s/DOWNLOADER=.*/DOWNLOADER=\"curl\"/" /etc/asl/config 
	  fi
	fi
  fi


	# Cloudlinux
	if uname -a |grep -q lve ; then
	 	perl -pi -e "s/FW_ENABLE_IPSET=.*/FW_ENABLE_IPSET=\"no\"/" /etc/asl/config 
		
	fi
} # end environment_detect

function install_components {
	
	# TODO: Normal
		# install mod_security

	# No ASL cpanel  / probably not needed. aum should handle this
	if [ $ASL -lt 1 -a "$PLATFORM" == "cpanel" ]; then
		# Disable scanner
 		perl -pi -e "s/MODSEC_99_SCANNER=.*/MODSEC_99_SCANNER=\"no\"/" /etc/asl/config 

                if [ -f /scripts/preeasyapache ]; then
                        if ! egrep -q "template-cpanel-preeasyapache-hook.*ASL_HOOK" /scripts/preeasyapache ; then
                                echo "/var/asl/data/templates/template-cpanel-preeasyapache-hook #ASL_HOOK" >> /scripts/preeasyapache

                        fi
                else
                        install -m0755 /var/asl/data/templates/template-cpanel-preeasyapache /scripts/preeasyapache
                fi
                if [ -f /scripts/posteasyapache ]; then
                        if ! egrep -q "template-cpanel-apache-hook.*ASL_HOOK" /scripts/posteasyapache ; then
                                echo "/var/asl/data/templates/template-cpanel-apache-hook #ASL_HOOK" >> /scripts/posteasyapache
                        fi
                else
                        install -m0755 /var/asl/data/templates/template-cpanel-posteasyapache /scripts/posteasyapache
                fi

                # Install modules 
		#if [ ! -f /usr/local/apache/modules/mod_security2.so -o ! -f /etc/httpd/conf.d/00_mod_security.conf ]; then
#
#			/usr/local/apache/bin/apachectl -l | grep -q unique
#			RETVAL=$?
##			if  [ ! "$RETVAL" = 0 ]; then
               	 		/scripts/easyapache --build
#			else
#				/var/asl/data/templates/template-cpanel-apache-hook module
#			fi
#		fi
	elif [ $ASL -lt 1 -a "$PLATFORM" == "directadmin" ]; then
		# Directadmin stub
		echo

	# Standard install
	elif [ $ASL -lt 1  ]; then
		if [ "$DIST" == "el5" ] && [ "$DIST" == "el6" ] && [ "$DIST" == "el7" ]; then
			# Installs only supported on EL5 & EL6
			if ! rpm -q mod_security >/dev/null; then
				echo -n "  Installing WAF Module:"
				if [ $LK_MODE -ge 1 ]; then 
					/usr/bin/yum -y --disablerepo="*" --enablerepo=tortix-common install mod_security >/dev/null 2>&1
				else
					/usr/bin/yum -y install mod_security >/dev/null 2>&1
				fi
				RETVAL=$?
				if [ $RETVAL -ge 1 ]; then
					echo "Failed"
					echo "  Error: WAF could not be installed. Configuration halted."
					exit 1

				else
					echo "Complete"
				fi
			fi
		fi
	
	fi

		
}

function feed_type {
	# TODO: key component
	if [ -f /etc/asl/license.key ]; then
		source /etc/asl/license.key
    		perl -pi -e "s/^FEED_SOURCE=.*/FEED_SOURCE=\"$FEED_SOURCE\"/"  /etc/asl/config
	fi
}

######## BEGIN ASL configuration ########
clear
echo
echo "Configuring Atomic Secured Linux (ASL) $ASL_VERSION for $HOSTNAME."

if [ "$CONFIGURED" == "no" ]; then
  echo "  This appears to be the first time you have run Atomic Secured Linux." 
  echo "  The following dialog will guide you through the  various settings "
  echo "  to configure ASL for your system"
  echo
  
  if grep -q "^USERNAME=\"USERNAME\"" /etc/asl/config ; then
    echo -n "Enter ASL Username: "
    read USERNAME < $SSH_TTY

    changepass "Enter Subscription Password" "no" "no"

    # Set the username and password to administer ASL in /etc/asl/config.
    perl -pi -e "s/^USERNAME=.*/USERNAME=\"$USERNAME\"/"  /etc/asl/config
    perl -pi -e "s/^PASSWORD=.*/PASSWORD=\"$PASSWORD\"/"  /etc/asl/config
  fi
fi

# Run the configuration routines for the basic ASL modules.
environment_detect
install_components
update_policy
if [ $ASL -ge 1 ]; then
	if [ ! $AUTO ]; then
		firewall_policy
	fi
	core_settings
	kernel_update
	kernelsettings
	php_settings
	#appinv_check
fi

# Set the 'configured' flag for ASL.
perl -pi -e "s/^CONFIGURED=.*/CONFIGURED=\"yes\"/"  /etc/asl/config 
# Initialize the DB
if [ -f /var/asl/bin/asl ]; then
	/var/asl/bin/asl --web_validate
fi



# Make a clean backup of the config file for restoration from ASL Web
cp /etc/asl/config /etc/asl/config.bak
chown tortix:root /etc/asl/config.bak
chmod 0600 /etc/asl/config.bak

echo
echo
echo


exit

# End of file.
